interface FormElement {
  label: string;
  key: string;
  type: string;
  value?: any;
  validation?: (value: any) => boolean;
}

function createForm(elements: FormElement[], callback: (values: { [key: string]: any }) => void) {
  // Create a form element
  const form = document.createElement("form");

  // Loop through the elements array
  for (const element of elements) {
    // Create container div
    const div = document.createElement("div");
    div.className = "form-input-group";
    // Create a label
    const label = document.createElement("label");
    label.innerHTML = element.label;

    // Create an input element
    const input = document.createElement("input");
    input.type = element.type;
    input.name = element.key;
    input.value = element.value || "";

    // Append the label and input to the form
    div.appendChild(label);
    div.appendChild(input);
    form.appendChild(div);
  }

  // Create a submit button
  const submit = document.createElement("input");
  submit.type = "submit";
  submit.value = "Submit";

  // Append the submit button to the form
  form.appendChild(submit);

  // Add event listener to validate inputs
  form.addEventListener("submit", (event) => {
    event.preventDefault();
    let valid = true;
    for (const element of elements) {
      if (element.validation) {
        valid = element.validation(form[element.key].value);
        if (!valid) {
          break;
        }
      }
    }
    if (valid) {
      console.log("Form Submitted");
      let formValues: { [key: string]: any } = {};
      for (const element of elements) {
        formValues[element.key] = form[element.key].value;
      }
      callback(formValues);
    } else {
      console.log("Form contains invalid data");
    }
  });
  // Return the form
  return form as HTMLFormElement;
}

function createTableSQL(elements: FormElement[], tableName: string) {
  let sql = `CREATE TABLE ${tableName} (
        id INT NOT NULL AUTO_INCREMENT,
        createdAt TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
        modifiedAt TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,`;
  for (const element of elements) {
    sql += `${element.key} ${element.type === 'number' ? 'INT' : 'VARCHAR(255)'} NOT NULL,`;
    if(element.type === "email"){
      sql += `INDEX (${element.key}),`;
    }
  }
  sql += `PRIMARY KEY (id)`;
  sql += `);`;
  return sql;
}

function createInsertSQL(elements: FormElement[], id?: number) {
  let sql = `INSERT INTO form_data (`;
  let values = "VALUES (";
  let update = " ON DUPLICATE KEY UPDATE ";
  for (const element of elements) {
    sql += `${element.key},`;
    values += `? ,`;
    update += `${element.key} = ? ,`;
  }
  sql = sql.slice(0, -1);
  values = values.slice(0, -1);
  update = update.slice(0, -1);
  if (id) {
    sql += `,id) ${values},?);`;
  } else {
    sql += `) ${values});`;
  }

  if (id) {
    sql += ` ${update}`;
  }
  return sql;
}


const elements: FormElement[] = [
  { label: "Name", key: "name", type: "text", validation: (value: string) => value.length > 0 },
  { label: "Age", key: "age", type: "number", validation: (value: number) => value > 0 },
  { label: "Email", key: "email", type: "email", validation: (value: string) => value.indexOf("@") != -1 },
];

const formElems: FormElement[] = [
  { label: "Label", key: "label", type: "text", validation: (value: string) => value.length > 0 },
  { label: "Input Key", key: "input_key", type: "text", validation: (value: string) => value.length > 0 },
  { label: "Input Type", key: "input_type", type: "text", validation: (value: string) => value.length > 0 },
];
const form = createForm(elements, (values) => {
  console.log(values);
});
const form2 = createForm(formElems, (values) => {
  console.log(values);
});
let app = document.querySelector("#app");

if (app) app.appendChild(form2);
if (app) app.appendChild(form);

const sql2 = createTableSQL(formElems, "forms");
console.log(sql2);
const sql = createTableSQL(elements, "users");
console.log(sql);

const sql3 = createInsertSQL(formElems, 1);
console.log(sql3);
const sql4 = createInsertSQL(elements, 1);
console.log(sql4);